# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from plone import api

import collective.printrss


class CollectivePrintrssLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=collective.printrss)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.printrss:default')
        api.user.create(email='test@imio.be', username='test')
        api.user.grant_roles(username='test', roles=['Site Administrator'])

COLLECTIVE_PRINTRSS_FIXTURE = CollectivePrintrssLayer()


COLLECTIVE_PRINTRSS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_PRINTRSS_FIXTURE,),
    name='CollectivePrintrssLayer:IntegrationTesting'
)


COLLECTIVE_PRINTRSS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_PRINTRSS_FIXTURE,),
    name='CollectivePrintrssLayer:FunctionalTesting'
)


COLLECTIVE_PRINTRSS_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_PRINTRSS_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectivePrintrssLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from collective.printrss import _
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.interface import invariant, Invalid


class ICollectivePrintrssLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IRssFeed(Interface):

    title = schema.TextLine(
        title=_(u"Title"),
        required=True,
    )

    description = schema.Text(
        title=_(u"Description"),
        required=False,
    )

    count = schema.Int(
        title=_(u'Number of items to display'),
        description=_(u'How many items to list.'),
        required=True,
        default=5)

    url = schema.TextLine(
        title=_(u'URL of RSS feed'),
        description=_(u'Link of the RSS feed to display.'),
        required=True,
        default=u'')

    url_main = schema.TextLine(
        title=_(u'URL of main page of external site'),
        description=_(u'If this field is completed, in index view, the title link of the rss feed will redirect to this url if not, you will be redirected to the homepage of the rss link site.'),
        required=False,
        default=u'')

    want_to_print_summary = schema.Bool(
        title=_(u'Print summary'),
        description=_(u'Do you want to print summary'),
        required=False,
        default=False)

    want_to_print_picture = schema.Bool(
        title=_(u'Print pictures'),
        description=_(u'Do you want to print picture'),
        required=False,
        default=False)

    timeout = schema.Int(
        title=_(u'Feed reload timeout'),
        description=_(u'Time in minutes after which the feed should be '
                      u'reloaded.'),
        required=True,
        default=100)

    @invariant
    def countInvariant(data):
        if data.count > 20:
            raise Invalid(_(u"Number of items must be <= 20!"))



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_ID
from zope.component import queryUtility
from zope.component import createObject
from plone.app.testing import setRoles
from plone.dexterity.interfaces import IDexterityFTI
from plone import api

from collective.printrss.testing import COLLECTIVE_PRINTRSS_INTEGRATION_TESTING  # noqa
from collective.printrss.interfaces import IRssFeed

import unittest2 as unittest


class rss_feedIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_PRINTRSS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='rss_feed')
        schema = fti.lookupSchema()
        self.assertEqual(IRssFeed, schema)

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='rss_feed')
        self.assertTrue(fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='rss_feed')
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(IRssFeed.providedBy(obj))

    def test_adding(self):
        self.portal.invokeFactory('rss_feed', 'rss_feed')
        self.assertTrue(
            IRssFeed.providedBy(self.portal['rss_feed'])
        )



from DateTime import DateTime
from DateTime.interfaces import DateTimeError

from plone.app.portlets.portlets.rss import RSSFeed
from plone.dexterity.content import Item

import feedparser

FEED_DATA = {}


class RichRSSFeed(RSSFeed):

    # __metaclass__ = feedparser

    @property
    def feed_version(self):
        # rss20 (atom)
        return self.rss_datas.version

    def __init__(self, url, timeout):
        self.rss_datas = feedparser.parse(url)
        RSSFeed.__init__(self, url, timeout)

    def _buildItemDict(self, item):
        link = item.links[0]["href"]
        itemdict = {
            "title": item.title,
            "url": link,
            "summary": item.get("description", ""),
            "image_url": self._search_for_picture(item),
        }
        if hasattr(item, "updated"):
            try:
                itemdict["updated"] = DateTime(item.updated)
            except DateTimeError:
                # It's okay to drop it because in the
                # template, this is checked with
                # ``exists:``
                pass
        return itemdict
        # myitem = item
        # otheritem = RSSFeed._buildItemDict(self, item)

    def _search_for_picture(self, item):
        image_url = None
        img_extensions = [".bmp", ".gif", ".jpg", ".png"]
        if hasattr(item, "links"):
            for link in item.links:
                if "href" in link and image_url is None:
                    image_url = (
                        link.href
                        if ("type" in link and "image" in link.type)
                        or any(x in link.href for x in img_extensions)
                        else None
                    )
        return image_url


class RssFeed(Item):
    @property
    def initializing(self):
        """should return True if deferred template should be displayed"""
        feed = self._getFeed()
        if not feed.loaded:
            return True
        if feed.needs_update:
            return True
        return False

    def deferred_update(self):
        """refresh data for serving via KSS"""
        feed = self._getFeed()
        feed.update()

    def update(self):
        """update data before rendering. We can not wait for KSS since users
        may not be using KSS."""
        self.deferred_update()

    def _getFeed(self):
        """return a feed object but do not update it"""
        feed = FEED_DATA.get(self.url, None)
        if feed is None:
            # create it
            feed = FEED_DATA[self.url] = RichRSSFeed(self.url, self.timeout)
        return feed

    @property
    def siteurl(self):
        """return url of site for portlet"""
        return self._getFeed().siteurl

    @property
    def feedlink(self):
        """return rss url of feed for portlet"""
        return self.url.replace("http://", "feed://")

    @property
    def feedAvailable(self):
        """checks if the feed data is available"""
        return self._getFeed().ok

    @property
    def items(self):
        if self._getFeed().needs_update:
            self._getFeed().update()
        return self._getFeed().items[: self.count]

    @property
    def enabled(self):
        return self._getFeed().ok
